// -*-c++-*-

/***************************************************************************
                          rcccoachmsg.h  -  Interface of the RoboCup coach
			                    language messages classes
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from 
                           Soccer Server's coach_lang_comp.h by the RoboCup
			   Soccer Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_COACH_MSG_H_
#define _RCC_COACH_MSG_H_

#pragma interface

#include <iostream>
#include <vector>
#include "rccstring.h"

#include "param.h"
#include "rcccoachtoken.h"
#include "rccobject.h"
#include "rccmemory.h"

/********* CoachStdMsg *******************/

class RCC_CoachStdMsg : public RCC_Object
{
public:
  enum coach_msg_t
  {
    CM_VOID,
    CM_META,
    CM_FREE,
    CM_INFO,
    CM_ADV,
    CM_DEF
  };

  RCC_CoachStdMsg() {}
  virtual ~RCC_CoachStdMsg() {}

  virtual std::ostream& print(std::ostream& out) const = 0;
  virtual std::ostream& printPretty(std::ostream& out, const RCC_String& line_header) const = 0;

  friend std::ostream& operator <<(std::ostream & os, RCC_CoachStdMsg& m)
    { return m.print(os); }

} ;


class RCC_CoachMetaMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachMetaMsg () {}
  ~RCC_CoachMetaMsg() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_CoachMetaMsg > msg_copy_p = new RCC_CoachMetaMsg ();
    for ( RCC_CoachMetaToken_Version::container::const_iterator i 
	    = ver_tokens.begin ();
	  i != ver_tokens.end (); ++i )
      {
	msg_copy_p->add ( ( *i )->copy ().staticCast < RCC_CoachMetaToken_Version > () );
      }
    return msg_copy_p;
  }

  std::ostream& 
  print ( std::ostream& out ) const;
  
  std::ostream& 
  printPretty ( std::ostream& out, const RCC_String& line_header ) const;
  
  void
  add ( RCC_Pointer < RCC_CoachMetaToken_Version > t ) 
  {
    tokens_.push_back ( t );
    ver_tokens.push_back ( t );
  }

  RCC_CoachMetaToken_Version::container&
  verTokens ()
  { return ver_tokens; }

  const RCC_CoachMetaToken_Version::container&
  verTokens () const
  { return ver_tokens; }

private:
  RCC_CoachMetaToken::container tokens_; //used for printing
  RCC_CoachMetaToken_Version::container ver_tokens;
  
} ;


/******* Freeform Message *********************/

class RCC_CoachFreeformMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachFreeformMsg ( const RCC_String& s = "" ) : str_ ( s ) {}
  ~RCC_CoachFreeformMsg () {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_CoachFreeformMsg ( str_ ); }

  std::ostream& print ( std::ostream& out ) const;
  std::ostream& printPretty ( std::ostream& out, const RCC_String& line_header ) const;
    
  RCC_String str () const { return str_; }
  RCC_String str ( const RCC_String& s ) { return str_ = s; }
 
private:
  RCC_String str_;
} ;



/******** Info Message ********************/
class RCC_CoachInfoMsg 
  : public RCC_CoachStdMsg 
{
public:
  RCC_CoachInfoMsg() {}
  ~RCC_CoachInfoMsg() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    // NOTE: The copy will *NOT* have the same print order as the original
    RCC_Pointer < RCC_CoachInfoMsg > msg_copy = new RCC_CoachInfoMsg ();
    for ( RCC_CoachInfoToken_Clear::container::const_iterator i 
	    = clear_tokens.begin ();
	  i != clear_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachInfoToken_Clear > () );
      }
    for ( RCC_CoachInfoToken_Rule::container::const_iterator i 
	    = rule_tokens.begin ();
	  i != rule_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachInfoToken_Rule > () );
      }
    return msg_copy;
  }

  std::ostream& 
  print ( std::ostream& out ) const;
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  void 
  add ( RCC_Pointer < RCC_CoachInfoToken_Clear > t )
  {
    tokens_.push_back ( t );
    clear_tokens.push_back ( t );
  }

  RCC_CoachInfoToken_Clear::container& 
  clearTokens ()
  { return clear_tokens; }

  const RCC_CoachInfoToken_Clear::container& 
  clearTokens () const
  { return clear_tokens; }
 
  void 
  add ( RCC_Pointer < RCC_CoachInfoToken_Rule > t )
  {
    tokens_.push_back ( t );
    rule_tokens.push_back ( t );
  }

  RCC_CoachInfoToken_Rule::container& 
  ruleTokens ()
  { return rule_tokens; }

  const RCC_CoachInfoToken_Rule::container& 
  ruleTokens () const
  { return rule_tokens; }
 
private:
  RCC_CoachInfoToken::container tokens_; //used for printing
  RCC_CoachInfoToken_Clear::container clear_tokens;
  RCC_CoachInfoToken_Rule::container rule_tokens;
  
};

/******** Advice Message ********************/
class RCC_CoachAdviceMsg 
  : public RCC_CoachStdMsg 
{
public:
  RCC_CoachAdviceMsg() {}
  ~RCC_CoachAdviceMsg() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    // NOTE: The copy will *NOT* have the same print order as the original
    RCC_Pointer < RCC_CoachAdviceMsg > msg_copy = new RCC_CoachAdviceMsg ();
    for ( RCC_CoachInfoToken_Clear::container::const_iterator i 
	    = clear_tokens.begin ();
	  i != clear_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachInfoToken_Clear > () );
      }
    for ( RCC_CoachInfoToken_Rule::container::const_iterator i 
	    = rule_tokens.begin ();
	  i != rule_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachInfoToken_Rule > () );
      }
    return msg_copy;
  }

  std::ostream& 
  print ( std::ostream& out ) const;
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  void 
  add ( RCC_Pointer < RCC_CoachInfoToken_Clear > t )
  {
    tokens_.push_back ( t );
    clear_tokens.push_back ( t );
  }

  RCC_CoachInfoToken_Clear::container& 
  clearTokens ()
  { return clear_tokens; }

  const RCC_CoachInfoToken_Clear::container& 
  clearTokens () const
  { return clear_tokens; }
 
  void 
  add ( RCC_Pointer < RCC_CoachInfoToken_Rule > t )
  {
    tokens_.push_back ( t );
    rule_tokens.push_back ( t );
  }

  RCC_CoachInfoToken_Rule::container& 
  ruleTokens ()
  { return rule_tokens; }

  const RCC_CoachInfoToken_Rule::container& 
  ruleTokens () const
  { return rule_tokens; }
 
private:
  RCC_CoachInfoToken::container tokens_; //used for printing
  RCC_CoachInfoToken_Clear::container clear_tokens;
  RCC_CoachInfoToken_Rule::container rule_tokens;
};


/******** Define Message ********************/
class RCC_CoachDefineMsg : public RCC_CoachStdMsg 
{
public:
  RCC_CoachDefineMsg () {}
  virtual ~RCC_CoachDefineMsg () {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    // NOTE: The copy will *NOT* have the same print order as the original
    RCC_Pointer < RCC_CoachDefineMsg > msg_copy = new RCC_CoachDefineMsg ();
    for ( RCC_CoachDefineToken_Condition::container::const_iterator i 
	    = cond_tokens.begin ();
	  i != cond_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachDefineToken_Condition > () );
      }
    for ( RCC_CoachDefineToken_Directive::container::const_iterator i 
	    = dir_tokens.begin ();
	  i != dir_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachDefineToken_Directive > () );
      }
    for ( RCC_CoachDefineToken_Region::container::const_iterator i 
	    = reg_tokens.begin ();
	  i != reg_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachDefineToken_Region > () );
      }
    for ( RCC_CoachDefineToken_Action::container::const_iterator i 
	    = act_tokens.begin ();
	  i != act_tokens.end (); ++i )
      {
	msg_copy->add ( ( *i )->copy ().staticCast < RCC_CoachDefineToken_Action > () );
      }
    return msg_copy;
  }

  std::ostream& 
  print ( std::ostream& out ) const;
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;
  
  void 
  add ( RCC_Pointer < RCC_CoachDefineToken_Condition > t )
  {
    tokens_.push_back ( t );
    cond_tokens.push_back ( t );
  }

  RCC_CoachDefineToken_Condition::container& 
  condTokens ()
  { return cond_tokens; }

  const RCC_CoachDefineToken_Condition::container& 
  condTokens () const
  { return cond_tokens; }
 
  void 
  add ( RCC_Pointer < RCC_CoachDefineToken_Directive > t )
  {
    tokens_.push_back ( t );
    dir_tokens.push_back ( t );
  }

  RCC_CoachDefineToken_Directive::container& 
  dirTokens ()
  { return dir_tokens; }

  const RCC_CoachDefineToken_Directive::container& 
  dirTokens () const
  { return dir_tokens; }
 
  void 
  add ( RCC_Pointer < RCC_CoachDefineToken_Region > t )
  {
    tokens_.push_back ( t );
    reg_tokens.push_back ( t );
  }

  RCC_CoachDefineToken_Region::container& 
  regTokens ()
  { return reg_tokens; }

  const RCC_CoachDefineToken_Region::container& 
  regTokens () const
  { return reg_tokens; }

  void 
  add ( RCC_Pointer < RCC_CoachDefineToken_Action > t )
  {
    tokens_.push_back ( t );
    act_tokens.push_back ( t );
  }

  RCC_CoachDefineToken_Action::container& 
  actTokens ()
  { return act_tokens; }

  const RCC_CoachDefineToken_Action::container& 
  actTokens () const
  { return act_tokens; }
  
private:
  RCC_CoachDefineToken::container tokens_; //use for printing

  RCC_CoachDefineToken_Condition::container 	cond_tokens;
  RCC_CoachDefineToken_Directive::container 	dir_tokens;
  RCC_CoachDefineToken_Region::container 	reg_tokens;
  RCC_CoachDefineToken_Action::container 	act_tokens;
  
};


#endif 
