// -*-c++-*-

/***************************************************************************
                          rcccond.h  -  Interface for RoboCup coach language
			                condition classes
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_COND_H_
#define _RCC_COND_H_

#pragma interface

#include <iostream>
#include <vector>
#include "rccstring.h"

#include "rccobject.h"
#include "rccregion.h"
#include "rccplaymode.h"
#include "rccside.h"
#include "rccmemory.h"
#include "rccexception.h"

class RCC_Condition 
  : public RCC_Object
{
public:
  enum cond_type_t {
    COND_VOID,
    COND_BOOL,
    COND_PPOS,
    COND_BPOS,
    COND_BOWN,
    COND_PM,
    COND_AND,
    COND_OR,
    COND_NOT,
    COND_NAMED
  };

  typedef RCC_Container < RCC_Condition > container;

  RCC_Condition() {}
  virtual ~RCC_Condition() {}

  virtual std::ostream& 
  print ( std::ostream& out ) const = 0;

  virtual std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const = 0;

  friend std::ostream& 
  operator<< ( std::ostream & os, 
	       RCC_Condition& c )
  { return c.print(os); }

  virtual bool 
  eval () const = 0;

  operator bool () const 
  { return eval (); } 
};

class RCC_CondBool;
class RCC_CondPlayerPosition;
class RCC_CondBallPosition;
class RCC_CondBallOwner;
class RCC_CondPlayMode;
class RCC_CondAnd;
class RCC_CondOr;
class RCC_CondNot;
class RCC_CondNamed;

class RCC_CondUnion
  : public RCC_Object
{
public:
  typedef RCC_Container < RCC_CondUnion > container;
  
  RCC_CondUnion ( RCC_Pointer < RCC_CondBool > cond )
    : cond_p ( cond ),
      c_bool_p ( cond ),
      cond_type ( RCC_Condition::COND_BOOL )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondPlayerPosition > cond )
    : cond_p ( cond ),
      c_ppos_p ( cond ),
      cond_type ( RCC_Condition::COND_PPOS )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondBallPosition > cond )
    : cond_p ( cond ),
      c_bpos_p ( cond ),
      cond_type ( RCC_Condition::COND_BPOS )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondBallOwner > cond )
    : cond_p ( cond ),
      c_bown_p ( cond ),
      cond_type ( RCC_Condition::COND_BOWN )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondPlayMode > cond )
    : cond_p ( cond ),
      c_pm_p ( cond ),
      cond_type ( RCC_Condition::COND_PM )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondAnd > cond )
    : cond_p ( cond ),
      c_and_p ( cond ),
      cond_type ( RCC_Condition::COND_AND )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondOr > cond )
    : cond_p ( cond ),
      c_or_p ( cond ),
      cond_type ( RCC_Condition::COND_OR )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondNot > cond )
    : cond_p ( cond ),
      c_not_p ( cond ),
      cond_type ( RCC_Condition::COND_NOT )
  {}

  RCC_CondUnion ( RCC_Pointer < RCC_CondNamed > cond )
    : cond_p ( cond ),
      c_named_p ( cond ),
      cond_type ( RCC_Condition::COND_NAMED )
  {}

  ~RCC_CondUnion () {}

  RCC_Pointer < RCC_Object >
  copy () const
  {
    switch ( cond_type )
      {
      case RCC_Condition::COND_BOOL:
	{
	  RCC_Pointer < RCC_CondBool > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondBool > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_PPOS:
	{
	  RCC_Pointer < RCC_CondPlayerPosition > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondPlayerPosition > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_BPOS:
	{
	  RCC_Pointer < RCC_CondBallPosition > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondBallPosition > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_BOWN:
	{
	  RCC_Pointer < RCC_CondBallOwner > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondBallOwner > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_PM:
	{
	  RCC_Pointer < RCC_CondPlayMode > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondPlayMode > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_AND:
	{
	  RCC_Pointer < RCC_CondAnd > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondAnd > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_OR:
	{
	  RCC_Pointer < RCC_CondOr > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondOr > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_NOT:
	{
	  RCC_Pointer < RCC_CondNot > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondNot > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      case RCC_Condition::COND_NAMED:
	{
	  RCC_Pointer < RCC_CondNamed > cond_copy = 
	    cond_p->copy ().staticCast < RCC_CondNamed > ();
	  return new RCC_CondUnion ( cond_copy );
	}

      default:
	return NULL;
      }
  }

  RCC_Condition::cond_type_t 
  type () const
  { return cond_type; }

  RCC_Pointer < RCC_CondBool > 
  setCond ( RCC_Pointer < RCC_CondBool > cond )
  {
    cond_p = cond;
    c_bool_p = cond;
    cond_type = RCC_Condition::COND_BOOL ;
    return cond;
  }

  RCC_Pointer < RCC_CondPlayerPosition > 
  setCond ( RCC_Pointer < RCC_CondPlayerPosition > cond )
  {
    cond_p = cond;
    c_ppos_p = cond;
    cond_type = RCC_Condition::COND_PPOS ;
    return cond;
  }

  RCC_Pointer < RCC_CondBallPosition > 
  setCond ( RCC_Pointer < RCC_CondBallPosition > cond )
  {
    cond_p = cond;
    c_bpos_p = cond;
    cond_type = RCC_Condition::COND_BPOS ;
    return cond;
  }

  RCC_Pointer < RCC_CondBallOwner > 
  setCond ( RCC_Pointer < RCC_CondBallOwner > cond )
  {
    cond_p = cond;
    c_bown_p = cond;
    cond_type = RCC_Condition::COND_BOWN ;
    return cond;
  }

  RCC_Pointer < RCC_CondPlayMode > 
  setCond ( RCC_Pointer < RCC_CondPlayMode > cond )
  {
    cond_p = cond;
    c_pm_p = cond;
    cond_type = RCC_Condition::COND_PM;
    return cond;
  }

  RCC_Pointer < RCC_CondAnd > 
  setCond ( RCC_Pointer < RCC_CondAnd > cond )
  {
    cond_p = cond;
    c_and_p = cond;
    cond_type = RCC_Condition::COND_AND;
    return cond;
  }

  RCC_Pointer < RCC_CondOr > 
  setCond ( RCC_Pointer < RCC_CondOr > cond )
  {
    cond_p = cond;
    c_or_p = cond;
    cond_type = RCC_Condition::COND_OR;
    return cond;
  }

  RCC_Pointer < RCC_CondNot > 
  setCond ( RCC_Pointer < RCC_CondNot > cond )
  {
    cond_p = cond;
    c_not_p = cond;
    cond_type = RCC_Condition::COND_NOT;
    return cond;
  }

  RCC_Pointer < RCC_CondNamed > 
  setCond ( RCC_Pointer < RCC_CondNamed > cond )
  {
    cond_p = cond;
    c_named_p = cond;
    cond_type = RCC_Condition::COND_NAMED;
    return cond;
  }


  RCC_Pointer < RCC_CondBool >
  condBool ()
  {
    if ( cond_type == RCC_Condition::COND_BOOL )
      return c_bool_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondPlayerPosition >
  condPPos ()
  {
    if ( cond_type == RCC_Condition::COND_PPOS )
      return c_ppos_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondBallPosition >
  condBPos ()
  {
    if ( cond_type == RCC_Condition::COND_BPOS )
      return c_bpos_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondBallOwner >
  condBOwn ()
  {
    if ( cond_type == RCC_Condition::COND_BOWN )
      return c_bown_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondPlayMode >
  condPM ()
  {
    if ( cond_type == RCC_Condition::COND_PM )
      return c_pm_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondAnd >
  condAnd ()
  {
    if ( cond_type == RCC_Condition::COND_AND )
      return c_and_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondOr >
  condOr ()
  {
    if ( cond_type == RCC_Condition::COND_OR )
      return c_or_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondNot >
  condNot ()
  {
    if ( cond_type == RCC_Condition::COND_NOT )
      return c_not_p;
    return NULL;
  }

  RCC_Pointer < RCC_CondNamed >
  condNamed ()
  {
    if ( cond_type == RCC_Condition::COND_NAMED )
      return c_named_p;
    return NULL;
  }

  RCC_Pointer < RCC_Condition >
  cond ()
  {
    return cond_p;
  }

  const RCC_Pointer < RCC_CondBool >
  condBool () const
  {
    if ( cond_type == RCC_Condition::COND_BOOL )
      return c_bool_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondPlayerPosition >
  condPPos () const
  {
    if ( cond_type == RCC_Condition::COND_PPOS )
      return c_ppos_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondBallPosition >
  condBPos () const
  {
    if ( cond_type == RCC_Condition::COND_BPOS )
      return c_bpos_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondBallOwner >
  condBOwn () const
  {
    if ( cond_type == RCC_Condition::COND_BOWN )
      return c_bown_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondPlayMode >
  condPM () const
  {
    if ( cond_type == RCC_Condition::COND_PM )
      return c_pm_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondAnd >
  condAnd () const
  {
    if ( cond_type == RCC_Condition::COND_AND )
      return c_and_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondOr >
  condOr () const
  {
    if ( cond_type == RCC_Condition::COND_OR )
      return c_or_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondNot >
  condNot () const
  {
    if ( cond_type == RCC_Condition::COND_NOT )
      return c_not_p;
    return NULL;
  }

  const RCC_Pointer < RCC_CondNamed >
  condNamed () const
  {
    if ( cond_type == RCC_Condition::COND_NAMED )
      return c_named_p;
    return NULL;
  }

  const RCC_Pointer < RCC_Condition >
  cond () const
  {
    return cond_p;
  }

private:
  RCC_Pointer < RCC_Condition > cond_p; //used to keep ref count +ve

  union
  {
    RCC_CondBool*		c_bool_p;
    RCC_CondPlayerPosition*	c_ppos_p;
    RCC_CondBallPosition*	c_bpos_p;
    RCC_CondBallOwner*		c_bown_p;
    RCC_CondPlayMode*		c_pm_p;
    RCC_CondAnd*		c_and_p;
    RCC_CondOr*			c_or_p;
    RCC_CondNot*		c_not_p;
    RCC_CondNamed*		c_named_p;
  };

  RCC_Condition::cond_type_t 	cond_type;
};



class RCC_CondRegion
{
public:
  RCC_CondRegion ( const RCC_Pointer < RCC_RegUnion >& r ) : reg_p ( r ) {}
  virtual ~RCC_CondRegion () {}

  /* class specific stuff */
  RCC_Pointer < RCC_RegUnion >& 
  region() 
  { return reg_p; }
  
  const RCC_Pointer < RCC_RegUnion >& 
  region() const 
  { return reg_p; }
  
  RCC_Pointer < RCC_RegUnion >& 
  region ( const RCC_Pointer < RCC_RegUnion >& r ) 
  { return reg_p = r; }

private:
  RCC_Pointer < RCC_RegUnion > reg_p;
};

class RCC_CondPlayer
{
public:
  RCC_CondPlayer ( const RCC_Side& s, 
		   const RCC_UnumSet& p )
    : side_ ( s ), 
    players ( p ) 
  {}

  virtual ~RCC_CondPlayer () {}

  /* class specific stuff */
  bool ourSide() const { return side_.ourSide (); }
  bool oppSide() const { return side_.oppSide (); }
  RCC_Side side () const { return side_; }
  RCC_Side side ( const RCC_Side& s ) { return side_ = s; }

  RCC_UnumSet& playerSet () { return players; }
  const RCC_UnumSet& constPlayerSet () const { return players; }

  void playerSet ( const RCC_UnumSet& p ) { players = p; }
  void addPlayer ( const int& i ) { players.add ( i ); }
  void removePlayer ( const int& i ) { players.remove ( i ); }
  void clearPlayers () { players.clear(); }

private:
  RCC_Side side_;
  RCC_UnumSet players;
};

class RCC_CondBool 
  : public RCC_Condition
{
public:
  RCC_CondBool ( const bool& b )
    : val ( b ) 
  {}
  
  ~RCC_CondBool () {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_CondBool ( val ); }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << ( val ? "(true)" : "(false)" ); }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << ( val ? "true" : "false" ) << std::endl; }

  bool eval () const { return val; }

private:
  bool val;
};


class RCC_CondPlayerPosition 
  : public RCC_Condition, 
    public RCC_CondPlayer, 
    public RCC_CondRegion
{
public:
  RCC_CondPlayerPosition ( const RCC_Side& s, const RCC_UnumSet& p, 
			   const int& min_players, const int& max_players, 
			   const RCC_Pointer < RCC_RegUnion > r )
    : RCC_CondPlayer ( s, p ),
      RCC_CondRegion ( r ), 
      min_ ( min_players ), 
      max_ ( max_players )
  {}

  ~RCC_CondPlayerPosition() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_CondPlayerPosition ( side (), 
					constPlayerSet (), 
					min_, 
					max_, 
					reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const;
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  bool eval () const 
  { 
    throw RCC_Exception ( "eval funtion undefined for RCC_CondPlayerPosition");
  } //HOOK: need to add real evaluation here

  int min () const { return min_; }
  int max () const { return max_; }
  int min ( const int& m ) { return min_ = m; }
  int max ( const int& m ) { return max_ = m; }


private:
  int min_;
  int max_;
};

class RCC_CondBallPosition 
  : public RCC_Condition,
    public RCC_CondRegion
{
public:
  RCC_CondBallPosition ( const RCC_Pointer < RCC_RegUnion >& r ) 
    : RCC_CondRegion ( r ) {}
  
  ~RCC_CondBallPosition() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_RegUnion > reg_copy;
    reg_copy.staticCast ( region ()->copy () );
    return new RCC_CondBallPosition ( reg_copy ); 
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { out << "(bpos " << *( region ()->reg () ) << ")"; return out; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { out << line_header << "is ball position at: " << std::endl;
  region ()->reg ()->printPretty ( out, line_header + " "); return out; }

  bool eval () const 
  { 
    throw RCC_Exception ( "eval funtion undefined for RCC_CondBallPosition");
  } //HOOK: need to add real evaluation here


};

class RCC_CondBallOwner 
  : public RCC_Condition, 
    public RCC_CondPlayer
{
public:
  RCC_CondBallOwner ( const RCC_Side& s, 
		      const RCC_UnumSet& p ) 
    : RCC_CondPlayer ( s, p ) 
  {}

  ~RCC_CondBallOwner() {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_CondBallOwner ( side (), constPlayerSet () ); }

  std::ostream& 
  print ( std::ostream& out ) const;

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  bool eval () const 
  { 
    throw RCC_Exception ( "eval funtion undefined for RCC_CondBallOwner" );
  } //HOOK: need to add real evaluation here


};

class RCC_CondPlayMode : public RCC_Condition
{
public:
  RCC_CondPlayMode ( const RCC_CoachPlayMode& pm ) : play_mode ( pm ) {}
  ~RCC_CondPlayMode() {}

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_CondPlayMode ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "(playm " << play_mode << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { 
    return out << line_header << "is play mode '"
	       << play_mode << "'" << std::endl; 
  }

  bool eval () const 
  { 
    throw RCC_Exception ( "eval funtion undefined for RCC_CondPlayMode" );
  } //HOOK: need to add real evaluation here

  /* class specific stuff */
  RCC_CoachPlayMode playMode() const { return play_mode; }
  void playMode ( const RCC_CoachPlayMode& pm ) { play_mode = pm; }


private:
  RCC_CoachPlayMode play_mode;
};



class RCC_CondConnected 
  : public RCC_Condition
{
public:
  RCC_CondConnected ( RCC_Pointer < RCC_CondUnion::container > conds )
    : conds_p ( conds )
  {}

  ~RCC_CondConnected () {}

  RCC_Pointer < RCC_CondUnion::container >
  conds () 
  { return conds_p; }

  const RCC_Pointer < RCC_CondUnion::container >
  conds () const  
  { return conds_p; }

  RCC_Pointer < RCC_CondUnion::container >
  conds ( RCC_Pointer < RCC_CondUnion::container > conds )
  { return conds_p = conds; }

private:
  RCC_Pointer < RCC_CondUnion::container > conds_p;
};

class RCC_CondAnd 
  : public RCC_CondConnected
{
public:
  RCC_CondAnd ( RCC_Pointer < RCC_CondUnion::container > conds ) 
  : RCC_CondConnected ( conds )
  {}

 ~RCC_CondAnd () {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_CondUnion::container > conds_copy 
      = conds ()->copy ().staticCast < RCC_CondUnion::container > ();
    return new RCC_CondAnd ( conds_copy );
  }

  std::ostream& 
  print ( std::ostream& out ) const;

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  bool eval () const;
};


class RCC_CondOr 
  : public RCC_CondConnected
{
public:
  RCC_CondOr ( RCC_Pointer < RCC_CondUnion::container > conds )
  : RCC_CondConnected ( conds ) 
  {}
  
  ~RCC_CondOr () {}

  RCC_Pointer < RCC_Object > copy () const 
  { 
    RCC_Pointer < RCC_CondUnion::container > conds_copy 
      = conds ()->copy ().staticCast < RCC_CondUnion::container > ();
    return new RCC_CondOr ( conds_copy );
  }

  std::ostream& 
  print ( std::ostream& out ) const;

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  bool eval () const;

};

class RCC_CondNot 
  : public RCC_Condition
{
public:
  RCC_CondNot ( RCC_Pointer < RCC_CondUnion > cu ) 
    : cu_p ( cu ) 
  {}

  ~RCC_CondNot() {}

  RCC_Pointer < RCC_Object > copy () const 
  {
    RCC_Pointer < RCC_CondUnion > cu_copy =
      cu_p->copy().staticCast < RCC_CondUnion > ();
    return new RCC_CondNot ( cu_copy );
  }

  std::ostream& 
  print ( std::ostream& out ) const 
  { 
    if ( cu_p != NULL && cu_p->cond () != NULL ) 
      out << "(not " << *cu_p->cond () << ")"; 
    return out; 
  }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  {
    if ( cu_p != NULL && cu_p->cond () != NULL )
      {
	out << line_header << "not " << std::endl;
	cu_p->cond ()->printPretty ( out, line_header + " +" );
      }
    return out; 
  }

  bool eval () const { return !cu_p->cond ()->eval (); }

private:
  RCC_Pointer < RCC_CondUnion > cu_p;

};



class RCC_CondNamed 
  : public RCC_Condition
{
public:
  RCC_CondNamed ( const RCC_String& n = "" ) 
    : name_ ( n ) 
  {}
  
  ~RCC_CondNamed() {}

  RCC_Pointer < RCC_Object > copy () const 
  { return new RCC_CondNamed ( *this ); }

  std::ostream& 
  print ( std::ostream& out ) const 
  { return out << "\"" << name_ << "\""; }

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { 
    return out << line_header << "condition named \"" 
	       << name_ << "\"" << std::endl;
  }

  bool 
  eval () const 
  { 
    throw RCC_Exception ( "eval function undefined for RCC_CondNamed"); 
  } //HOOK: add real evlauation

  /* class specific */
  RCC_String 
  name () 
  { return name_; }

  RCC_String 
  name ( const RCC_String& n ) 
  { return name_ = n; }

private:
  RCC_String name_;

};

#endif
