// -*-c++-*-

/***************************************************************************
                          rccerror.h  -  Interface for RoboCup error message
			                 classes.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _RCC_ERROR_H_
#define _RCC_ERROR_H_

#pragma interface

#include "rccplaymode.h"
#include <iostream>
#include "types.h"
#include "rccsensordata.h"
#include "rccstring.h"

class RCC_Error : public RCC_SensorData
{
public:
  static const RCC_String NAMES[];

  enum error_t {
    reconnect,
    no_more_team_or_player_or_goalie,
    no_such_team_or_already_have_coach,
    too_many_moves,
    unknown_command,
    illegal_command_form,
    say_message_too_long,
    illegal_mode,
    illegal_object_form,
    said_too_many_freeform_messages,
    cannot_say_freeform_while_playon,
    said_too_many_meta_messages,
    said_too_many_advice_messages,
    said_too_many_info_messages,
    said_too_many_define_messages,
    could_not_parse_say,
    said_too_many_messages
  };

  RCC_Error ( const RCC_Error::error_t& e, const int& tr = -1 )
    : error_ ( e ),
      time_recv ( tr )
  {}

  ~RCC_Error () {}

  RCC_Pointer < RCC_Object > copy () const
  { return new RCC_Error ( *this ); }

  RCC_Error::error_t error () const { return error_; }
  RCC_Error::error_t error ( const RCC_Error::error_t& e ) { return error_ = e; }

  int timeRecv () const { return time_recv; }
  int timeRecv ( const int& tr ) { return time_recv = tr; }

  std::ostream& print ( std::ostream& o ) const
  { return print ( o, NEUTRAL ); }

  std::ostream& print ( std::ostream& o, const Side& ) const
  {
    o << RCC_Error::NAMES [ error_ ];
    if ( error_ == RCC_Error::say_message_too_long )
      o << " " << time_recv;
    return o;
  }

private:
  RCC_Error::error_t error_;
  int time_recv;
};

#endif
