// -*-c++-*-

/***************************************************************************
                          rccfsobject.h  -  Interfaces and implemataion of
			                     for full state objects.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_FS_OBJECT_H_
#define _RCC_FS_OBJECT_H_

#include "rccobject.h"
#include "rccvector2d.h"
#include "rccside.h"
#include "rcccontainer.h"

class RCC_FSObject
  : public RCC_Object
{
public:
  typedef RCC_Container < RCC_FSObject > container;

  RCC_FSObject ( const RCC_Vector2d& pos )
    : abs_pos ( pos )
  {}

  virtual ~RCC_FSObject () {}

  RCC_Vector2d 
  absPos () const
  { return abs_pos; }

  RCC_Vector2d 
  absPos ( const RCC_Vector2d& pos )
  { return abs_pos = pos; }

private:
  RCC_Vector2d abs_pos;
};




class RCC_FSMobObject
  : public RCC_FSObject
{
public:
  RCC_FSMobObject ( const RCC_Vector2d& pos,
		    const RCC_Vector2d& vel )
    : RCC_FSObject ( pos ), 
    abs_vel ( vel )
  {}

  virtual ~RCC_FSMobObject () {}
 
  RCC_Vector2d
  absVel () const
  { return abs_vel; }

  RCC_Vector2d
  absVel ( const RCC_Vector2d& vel )
  { return abs_vel = vel; }

  std::ostream&
  print ( std::ostream& o )
  {
    return o << "(ball " << absPos ().x () << " " << absPos ().y () << " "
	     << abs_vel.x () << " " << abs_vel.y () << ")";
  }
    
private:
  RCC_Vector2d abs_vel;
};




class RCC_FSBall
  : public RCC_FSMobObject
{
public:
  typedef RCC_Container < RCC_FSBall > container;

  RCC_FSBall ( const RCC_Vector2d& pos,
	       const RCC_Vector2d& vel )
    : RCC_FSMobObject ( pos, vel )
  {}

  ~RCC_FSBall () {}

  RCC_Pointer < RCC_Object >
  copy () const
  { return new RCC_FSBall ( *this ); }

};



class RCC_FSPlayer
  : public RCC_FSMobObject
{
public:
  typedef RCC_Container < RCC_FSPlayer > container;

  RCC_FSPlayer ( const RCC_Side& s,
		 const int un,
		 const RCC_Vector2d& pos,
		 const RCC_Vector2d& vel,
		 const RCC_Angle& ba,
		 const RCC_Angle& ha,
		 const double& stam,
		 const double& eff,
		 const double& rec )
    : RCC_FSMobObject ( pos, vel ),
      side_ ( s ),
      unum ( un ),
      body_angle ( ba ),
      head_angle ( ha ),
      stamina_ ( stam ),
      effort_ ( eff ),
      recovery_ ( rec )
  {}

  ~RCC_FSPlayer () {}

  RCC_Pointer < RCC_Object >
  copy () const
  { return new RCC_FSPlayer ( *this ); }

  RCC_Side
  side () const
  { return side_; }

  RCC_Side
  side ( const RCC_Side& s )
  { return side_ = s; }

  int
  uNum () const
  { return unum; }

  int
  uNum ( const int& un )
  { return unum = un; }

  RCC_Angle
  bodyAngle () const
  { return body_angle; }

  RCC_Angle
  bodyAngle ( const RCC_Angle& ang )
  { return body_angle = ang; }

  RCC_Angle
  headAngle () const
  { return head_angle; }

  RCC_Angle
  headAngle ( const RCC_Angle& ang )
  { return head_angle = ang; }

  double
  stamina () const
  { return stamina_; }

  double
  stamina ( const double& stam )
  { return stamina_ = stam; }

  double
  effort () const
  { return effort_; }

  double
  effort ( const double& eff )
  { return effort_ = eff; }

  double
  recovery () const
  { return recovery_; }

  double
  recovery ( const double& rec )
  { return recovery_ = rec; }

  std::ostream&
  print ( std::ostream& o )
  {
    return o << "(" << side_ << " " 
	     << unum << " " 
	     << absPos ().x () << " " 
	     << absPos ().y () << " " 
	     << absVel ().x () << " " 
	     << absVel ().y () << " " 
	     << body_angle << " " 
	     << head_angle << " "
	     << stamina_ << " " 
	     << effort_ << " " 
	     << recovery_ << ")";
  }

private:
  RCC_Side side_;
  int unum;
  RCC_Angle body_angle;
  RCC_Angle head_angle;
  double stamina_;
  double effort_;
  double recovery_;
};


#endif
