// -*-c++-*-

/***************************************************************************
                          rccfullstate.h  -  Interface for RoboCup full
			                     state messages.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_FULL_STATE_H_
#define _RCC_FULL_STATE_H_

//#pragma interface
#include "rccsensordata.h"
#include "rccfsobject.h"
#include "rccplaymode.h"
#include "rccviewmode.h"
#include "rccscore.h"

class RCC_FullState
  : public RCC_SensorData
{
public:
  RCC_FullState ( int t,
		  RCC_Pointer < RCC_StadPlayMode > pm,
		  RCC_Pointer < RCC_ViewMode > vm,
		  RCC_Pointer < RCC_Score > s,
		  RCC_Pointer < RCC_FSBall > b,
		  RCC_Pointer < RCC_FSPlayer::container > p ) 
    : fs_time ( t ),
      pm_p ( pm ),
      vm_p ( vm ),
      score_p ( s ),
      ball_p ( b ),
      players_p ( p )
  {}

  ~RCC_FullState () {}

  RCC_Pointer < RCC_Object >
  copy () const
  {
    RCC_Pointer < RCC_StadPlayMode > pm_copy 
      = pm_p->copy ().staticCast < RCC_StadPlayMode > ();
    RCC_Pointer < RCC_ViewMode > vm_copy
      = vm_p->copy ().staticCast < RCC_ViewMode > ();
    RCC_Pointer < RCC_Score > score_copy
      = score_p->copy ().staticCast < RCC_Score > ();
    RCC_Pointer < RCC_FSBall > ball_copy
      = ball_p->copy ().staticCast < RCC_FSBall > ();
    RCC_Pointer < RCC_FSPlayer::container > players_copy
      = players_p->copy ().staticCast < RCC_FSPlayer::container > ();

    return new RCC_FullState ( fs_time, 
			       pm_copy, 
			       vm_copy, 
			       score_copy, 
			       ball_copy, 
			       players_copy );
  }

  std::ostream&
  print ( std::ostream& o ) const
  { return o << "(fs)" << std::endl; }

  std::ostream&
  print ( std::ostream& o, const Side& ) const
  { return print ( o ); }

  int
  time () const
  { return fs_time; }

  int
  time ( const int& t )
  { return fs_time = t; }

  RCC_Pointer < RCC_StadPlayMode >
  playMode ()
  { return pm_p; }

  const RCC_Pointer < RCC_StadPlayMode >
  playMode () const
  { return pm_p; }

  RCC_Pointer < RCC_StadPlayMode >
  playMode ( RCC_Pointer < RCC_StadPlayMode > pm )
  { return pm_p = pm; }

  RCC_Pointer < RCC_ViewMode >
  viewMode ()
  { return vm_p; }

  const RCC_Pointer < RCC_ViewMode >
  viewMode () const
  { return vm_p; }

  RCC_Pointer < RCC_ViewMode >
  viewMode ( RCC_Pointer < RCC_ViewMode > vm )
  { return vm_p = vm; }

  RCC_Pointer < RCC_Score >
  score ()
  { return score_p; }

  const RCC_Pointer < RCC_Score >
  score () const
  { return score_p; }

  RCC_Pointer < RCC_Score >
  score ( RCC_Pointer < RCC_Score > s )
  { return score_p = s; }

  RCC_Pointer < RCC_FSBall >
  ball ()
  { return ball_p; }

  const RCC_Pointer < RCC_FSBall >
  ball () const
  { return ball_p; }

  RCC_Pointer < RCC_FSBall >
  ball ( RCC_Pointer < RCC_FSBall > b )
  { return ball_p = b; }

  RCC_Pointer < RCC_FSPlayer::container >
  players ()
  { return players_p; }

  const RCC_Pointer < RCC_FSPlayer::container >
  players () const
  { return players_p; }

  RCC_Pointer < RCC_FSPlayer::container >
  players ( RCC_Pointer < RCC_FSPlayer::container > plays )
  { return players_p = plays; }

private:
  int fs_time;
  RCC_Pointer < RCC_StadPlayMode > pm_p;
  RCC_Pointer < RCC_ViewMode > vm_p;
  RCC_Pointer < RCC_Score > score_p;
  RCC_Pointer < RCC_FSBall > ball_p;
  RCC_Pointer < RCC_FSPlayer::container > players_p;
};


#endif
