// -*-c++-*-

/***************************************************************************
                          rccplayerparm.h  -  Interface for RoboCup player
			                      parameter message
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_PLAYER_PARAM_H_
#define _RCC_PLAYER_PARAM_H_

#pragma interface

#include <iostream>
#include "rccsensordata.h"

class RCC_PlayerParam : public RCC_SensorData
{
public:
  RCC_PlayerParam ( const int& player_types,
		    const int& subs_max,
		    const int& pt_max,
  
		    const double& player_speed_max_delta_min,
		    const double& player_speed_max_delta_max,
		    const double& stamina_inc_max_delta_factor,
    
		    const double& player_decay_delta_min,
		    const double& player_decay_delta_max,
		    const double& inertia_moment_delta_factor,

		    const double& dash_power_rate_delta_min,
		    const double& dash_power_rate_delta_max,
		    const double& player_size_delta_factor,

		    const double& kickable_margin_delta_min,
		    const double& kickable_margin_delta_max,
		    const double& kick_rand_delta_factor,

		    const double& extra_stamina_delta_min,
		    const double& extra_stamina_delta_max,
		    const double& effort_max_delta_factor,
		    const double& effort_min_delta_factor );

  ~RCC_PlayerParam () {}

  RCC_Pointer < RCC_Object > copy () const { return new RCC_PlayerParam ( *this ); }
   
  friend std::ostream& operator<< (std::ostream& o, const RCC_PlayerParam& sp)
  {
    return sp.print ( o );
  }

  std::ostream& print ( std::ostream& o ) const;
  std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 

  int playerTypes () const { return player_types; }
  int subsMax () const { return subs_max; }
  int playerTypeMax () const { return pt_max; }
  
  double playerSpeedMaxDeltaMin () const { return player_speed_max_delta_min; }
  double playerSpeedMaxDeltaMax () const { return player_speed_max_delta_max; }
  double staminaIncMaxDeltaFactor () const { return stamina_inc_max_delta_factor; }
    
  double playerDecayDeltaMin () const { return player_decay_delta_min; }
  double playerDecayDeltaMax () const { return player_decay_delta_max; }
  double inertiaMomentDeltaFactor () const { return inertia_moment_delta_factor; }

  double dashPowerRateDeltaMin () const { return dash_power_rate_delta_min; }
  double dashPowerRateDeltaMax () const { return dash_power_rate_delta_max; }
  double playerSizeDeltaFactor () const { return player_size_delta_factor; }

  double kickableMarginDeltaMin () const { return kickable_margin_delta_min; }
  double kickableMarginDeltaMax () const { return kickable_margin_delta_max; }
  double kickRandDeltaFactor () const { return kick_rand_delta_factor; }

  double extraStaminaDeltaMin () const { return extra_stamina_delta_min; }
  double extraStaminaDeltaMax () const { return extra_stamina_delta_max; }
  double effortMaxDeltaFactor () const { return effort_max_delta_factor; }
  double effortMinDeltaFactor () const { return effort_min_delta_factor; }

private:
  int player_types;
  int subs_max;
  int pt_max;
  
  double player_speed_max_delta_min;
  double player_speed_max_delta_max;
  double stamina_inc_max_delta_factor;
    
  double player_decay_delta_min;
  double player_decay_delta_max;
  double inertia_moment_delta_factor;

  double dash_power_rate_delta_min;
  double dash_power_rate_delta_max;
  double player_size_delta_factor;

  double kickable_margin_delta_min;
  double kickable_margin_delta_max;
  double kick_rand_delta_factor;

  double extra_stamina_delta_min;
  double extra_stamina_delta_max;
  double effort_max_delta_factor;
  double effort_min_delta_factor;
};


#endif
