// -*-c++-*-

/***************************************************************************
                          rccregion.h  -  Interface for RoboCup coach
			                  language region classes
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.  Modified from Soccer
                           Server's coach_lang_comp.h by the RoboCup Soccer
			   Server Maintainance Group.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_REGION_H_
#define _RCC_REGION_H_

#pragma interface

#include <iostream>
#include <vector>
#include "rccstring.h"

#include "rccangle.h"
#include "rccvector2d.h"
#include "param.h"
#include "rccside.h"
#include "rccobject.h"
#include "rccmemory.h"
#include "rccexception.h"
#include "rcccontainer.h"

class RCC_Region 
  : public RCC_Object
{
public:
  typedef RCC_Container < RCC_Region > container;

  RCC_Region() {}
  virtual ~RCC_Region() {};

  virtual std::ostream& print (std::ostream& out ) const = 0;
  virtual std::ostream& printPretty (std::ostream& out, const RCC_String& line_header ) const = 0;
  friend std::ostream& operator << (std::ostream & os, const RCC_Region& r )
    { return r.print(os); } 

  virtual RCC_Pointer < RCC_Object > copy() const = 0;
} ;

class RCC_RegNull;
class RCC_RegAbsPoint;
class RCC_RegBallPoint;
class RCC_RegPlayerPoint;
class RCC_RegRelPoint;
class RCC_RegQuad;
class RCC_RegArc;
class RCC_RegMerged;
class RCC_RegNamed;

class RCC_RegUnion
  : public RCC_Object
{
public:
  typedef RCC_Container < RCC_RegUnion > container;

  enum type_t
  {
    REG_VOID,
    REG_NULL,
    REG_ABS,
    REG_BALL,
    REG_PLAYER,
    REG_REL,
    REG_QUAD,
    REG_ARC,
    REG_MERGED,
    REG_NAMED
  };

  RCC_RegUnion ( RCC_Pointer < RCC_RegNull > reg )
    : reg_p ( reg ),
      reg_null_p ( reg ),
      type_ ( RCC_RegUnion::REG_NULL )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegAbsPoint > reg )
    : reg_p ( reg ),
      reg_abs_p ( reg ),
      type_ ( RCC_RegUnion::REG_ABS )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegBallPoint > reg )
    : reg_p ( reg ),
      reg_ball_p ( reg ),
      type_ ( RCC_RegUnion::REG_BALL )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegPlayerPoint > reg )
    : reg_p ( reg ),
      reg_player_p ( reg ),
      type_ ( RCC_RegUnion::REG_PLAYER )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegRelPoint > reg )
    : reg_p ( reg ),
      reg_rel_p ( reg ),
      type_ ( RCC_RegUnion::REG_REL )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegQuad > reg )
    : reg_p ( reg ),
      reg_quad_p ( reg ),
      type_ ( RCC_RegUnion::REG_QUAD )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegArc > reg )
    : reg_p ( reg ),
      reg_arc_p ( reg ),
      type_ ( RCC_RegUnion::REG_ARC )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegMerged > reg )
    : reg_p ( reg ),
      reg_merged_p ( reg ),
      type_ ( RCC_RegUnion::REG_MERGED )
  {}

  RCC_RegUnion ( RCC_Pointer < RCC_RegNamed > reg )
    : reg_p ( reg ),
      reg_named_p ( reg ),
      type_ ( RCC_RegUnion::REG_NAMED )
  {}

  ~RCC_RegUnion () {}

  RCC_Pointer < RCC_Object >
  copy () const
  {
    switch ( type_ )
      {
      case RCC_RegUnion::REG_NULL:
	{
	  RCC_Pointer < RCC_RegNull > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegNull > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_ABS:
	{
	  RCC_Pointer < RCC_RegAbsPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegAbsPoint > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_BALL:
	{
	  RCC_Pointer < RCC_RegBallPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegBallPoint > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_PLAYER:
	{
	  RCC_Pointer < RCC_RegPlayerPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegPlayerPoint > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_REL:
	{
	  RCC_Pointer < RCC_RegRelPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegRelPoint > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_QUAD:
	{
	  RCC_Pointer < RCC_RegQuad > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegQuad > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_ARC:
	{
	  RCC_Pointer < RCC_RegArc > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegArc > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_MERGED:
	{
	  RCC_Pointer < RCC_RegMerged > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegMerged > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      case RCC_RegUnion::REG_NAMED:
	{
	  RCC_Pointer < RCC_RegNamed > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegNamed > ();
	  return new RCC_RegUnion ( reg_copy ); 
	}
      default:
	return NULL;
      };
  }

  RCC_RegUnion::type_t
  type () const
  {
    return type_;
  }

  RCC_Pointer < RCC_RegNull >
  setReg ( RCC_Pointer < RCC_RegNull > reg )
  {
    reg_p = reg;
    reg_null_p = reg;
    type_ = RCC_RegUnion::REG_NULL;
    return reg;
  }

  RCC_Pointer < RCC_RegAbsPoint >
  setReg ( RCC_Pointer < RCC_RegAbsPoint > reg )
  {
    reg_p = reg;
    reg_abs_p = reg;
    type_ = RCC_RegUnion::REG_ABS;
    return reg;
  }

  RCC_Pointer < RCC_RegBallPoint >
  setReg ( RCC_Pointer < RCC_RegBallPoint > reg )
  {
    reg_p = reg;
    reg_ball_p = reg;
    type_ = RCC_RegUnion::REG_BALL;
    return reg;
  }

  RCC_Pointer < RCC_RegPlayerPoint >
  setReg ( RCC_Pointer < RCC_RegPlayerPoint > reg )
  {
    reg_p = reg;
    reg_player_p = reg;
    type_ = RCC_RegUnion::REG_PLAYER;
    return reg;
  }

  RCC_Pointer < RCC_RegRelPoint >
  setReg ( RCC_Pointer < RCC_RegRelPoint > reg )
  {
    reg_p = reg;
    reg_rel_p = reg;
    type_ = RCC_RegUnion::REG_REL;
    return reg;
  }

  RCC_Pointer < RCC_RegQuad >
  setReg ( RCC_Pointer < RCC_RegQuad > reg )
  {
    reg_p = reg;
    reg_quad_p = reg;
    type_ = RCC_RegUnion::REG_QUAD;
    return reg;
  }

  RCC_Pointer < RCC_RegArc >
  setReg ( RCC_Pointer < RCC_RegArc > reg )
  {
    reg_p = reg;
    reg_arc_p = reg;
    type_ = RCC_RegUnion::REG_ARC;
    return reg;
  }
 
  RCC_Pointer < RCC_RegMerged >
  setReg ( RCC_Pointer < RCC_RegMerged > reg )
  {
    reg_p = reg;
    reg_merged_p = reg;
    type_ = RCC_RegUnion::REG_MERGED;
    return reg;
  }

  RCC_Pointer < RCC_RegNamed >
  setReg ( RCC_Pointer < RCC_RegNamed > reg )
  {
    reg_p = reg;
    reg_named_p = reg;
    type_ = RCC_RegUnion::REG_NAMED;
    return reg;
  }

  RCC_Pointer < RCC_RegNull >
  regNull ()
  {
    if ( type_ == RCC_RegUnion::REG_NULL )
      return reg_null_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegNull >
  regNull () const
  {
    if ( type_ == RCC_RegUnion::REG_NULL )
      return reg_null_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegAbsPoint >
  regAbs ()
  {
    if ( type_ == RCC_RegUnion::REG_ABS )
      return reg_abs_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegAbsPoint >
  regAbs () const
  {
    if ( type_ == RCC_RegUnion::REG_ABS )
      return reg_abs_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegBallPoint >
  regBall ()
  {
    if ( type_ == RCC_RegUnion::REG_BALL )
      return reg_ball_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegBallPoint >
  regBall () const
  {
    if ( type_ == RCC_RegUnion::REG_BALL )
      return reg_ball_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegPlayerPoint >
  regPlayer ()
  {
    if ( type_ == RCC_RegUnion::REG_PLAYER )
      return reg_player_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegPlayerPoint >
  regPlayer () const
  {
    if ( type_ == RCC_RegUnion::REG_PLAYER )
      return reg_player_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegRelPoint >
  regRel ()
  {
    if ( type_ == RCC_RegUnion::REG_REL )
      return reg_rel_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegRelPoint >
  regRel () const
  {
    if ( type_ == RCC_RegUnion::REG_REL )
      return reg_rel_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegQuad >
  regQuad ()
  {
    if ( type_ == RCC_RegUnion::REG_QUAD )
      return reg_quad_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegQuad >
  regQuad () const
  {
    if ( type_ == RCC_RegUnion::REG_QUAD )
      return reg_quad_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegArc >
  regArc ()
  {
    if ( type_ == RCC_RegUnion::REG_ARC )
      return reg_arc_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegArc >
  regArc () const
  {
    if ( type_ == RCC_RegUnion::REG_ARC )
      return reg_arc_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegMerged >
  regMerged ()
  {
    if ( type_ == RCC_RegUnion::REG_MERGED )
      return reg_merged_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegMerged >
  regMerged () const
  {
    if ( type_ == RCC_RegUnion::REG_MERGED )
      return reg_merged_p;
    return NULL;
  }
 
  RCC_Pointer < RCC_RegNamed >
  regNamed ()
  {
    if ( type_ == RCC_RegUnion::REG_NAMED )
      return reg_named_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegNamed >
  regNamed () const
  {
    if ( type_ == RCC_RegUnion::REG_NAMED )
      return reg_named_p;
    return NULL;
  }

  RCC_Pointer < RCC_Region >
  reg ()
  {
    return reg_p;
  }

  const RCC_Pointer < RCC_Region >
  reg () const
  {
    return reg_p;
  }



private:
  RCC_Pointer < RCC_Region > reg_p; // used to keep ref count above zero
  union
  {
    RCC_RegNull*	reg_null_p;
    RCC_RegAbsPoint*	reg_abs_p;
    RCC_RegBallPoint*	reg_ball_p;
    RCC_RegPlayerPoint*	reg_player_p;
    RCC_RegRelPoint*	reg_rel_p;
    RCC_RegQuad*	reg_quad_p;
    RCC_RegArc*		reg_arc_p;
    RCC_RegMerged*	reg_merged_p;
    RCC_RegNamed*	reg_named_p;
  };

  RCC_RegUnion::type_t	type_;
};



class RCC_RegNull 
  : public RCC_Region
{
public:
  RCC_RegNull() {}
  ~RCC_RegNull() {}

  std::ostream& 
  print(std::ostream& out) const 
  { return out << "(null)"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const
  { return out << line_header << "null region" << std::endl; }
      
  RCC_Pointer < RCC_Object > 
  copy() const 
  { return new RCC_RegNull (); }
} ;



class RCC_RegPoint 
  : public RCC_Region
{
public:
  virtual ~RCC_RegPoint() {}

  friend std::ostream& operator << (std::ostream & os, const RCC_RegPoint& r )
    { return r.print(os); } 

  /* class specific */
    virtual std::ostream& printSimple ( std::ostream& out ) const = 0;

} ;



class RCC_RegPointUnion
  : public RCC_Object
{
public:
  typedef RCC_Container < RCC_RegPoint > container;

  enum type_t
  {
    REG_VOID,
    REG_ABS,
    REG_BALL,
    REG_PLAYER,
    REG_REL
  };

  RCC_RegPointUnion ( RCC_Pointer < RCC_RegAbsPoint > reg )
    : reg_p ( reg ),
      reg_abs_p ( reg ),
      type_ ( RCC_RegPointUnion::REG_ABS )
  {}

  RCC_RegPointUnion ( RCC_Pointer < RCC_RegBallPoint > reg )
    : reg_p ( reg ),
      reg_ball_p ( reg ),
      type_ ( RCC_RegPointUnion::REG_BALL )
  {}

  RCC_RegPointUnion ( RCC_Pointer < RCC_RegPlayerPoint > reg )
    : reg_p ( reg ),
      reg_player_p ( reg ),
      type_ ( RCC_RegPointUnion::REG_PLAYER )
  {}

  RCC_RegPointUnion ( RCC_Pointer < RCC_RegRelPoint > reg )
    : reg_p ( reg ),
      reg_rel_p ( reg ),
      type_ ( RCC_RegPointUnion::REG_REL )
  {}

  ~RCC_RegPointUnion () {}

  RCC_Pointer < RCC_Object >
  copy () const
  {
    switch ( type_ )
      {
      case RCC_RegPointUnion::REG_ABS:
	{
	  RCC_Pointer < RCC_RegAbsPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegAbsPoint > ();
	  return new RCC_RegPointUnion ( reg_copy ); 
	}
      case RCC_RegPointUnion::REG_BALL:
	{
	  RCC_Pointer < RCC_RegBallPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegBallPoint > ();
	  return new RCC_RegPointUnion ( reg_copy ); 
	}
      case RCC_RegPointUnion::REG_PLAYER:
	{
	  RCC_Pointer < RCC_RegPlayerPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegPlayerPoint > ();
	  return new RCC_RegPointUnion ( reg_copy ); 
	}
      case RCC_RegPointUnion::REG_REL:
	{
	  RCC_Pointer < RCC_RegRelPoint > reg_copy = 
	    reg_p->copy ().staticCast < RCC_RegRelPoint > ();
	  return new RCC_RegPointUnion ( reg_copy ); 
	}
      default:
	return NULL;
      };
  }

  RCC_RegPointUnion::type_t
  type () const
  {
    return type_;
  }

  RCC_Pointer < RCC_RegAbsPoint >
  setReg ( RCC_Pointer < RCC_RegAbsPoint > reg )
  {
    reg_p = reg;
    reg_abs_p = reg;
    type_ = RCC_RegPointUnion::REG_ABS;
    return reg;
  }

  RCC_Pointer < RCC_RegBallPoint >
  setReg ( RCC_Pointer < RCC_RegBallPoint > reg )
  {
    reg_p = reg;
    reg_ball_p = reg;
    type_ = RCC_RegPointUnion::REG_BALL;
    return reg;
  }

  RCC_Pointer < RCC_RegPlayerPoint >
  setReg ( RCC_Pointer < RCC_RegPlayerPoint > reg )
  {
    reg_p = reg;
    reg_player_p = reg;
    type_ = RCC_RegPointUnion::REG_PLAYER;
    return reg;
  }

  RCC_Pointer < RCC_RegRelPoint >
  setReg ( RCC_Pointer < RCC_RegRelPoint > reg )
  {
    reg_p = reg;
    reg_rel_p = reg;
    type_ = RCC_RegPointUnion::REG_REL;
    return reg;
  }

  RCC_Pointer < RCC_RegAbsPoint >
  regAbs ()
  {
    if ( type_ == RCC_RegPointUnion::REG_ABS )
      return reg_abs_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegAbsPoint >
  regAbs () const
  {
    if ( type_ == RCC_RegPointUnion::REG_ABS )
      return reg_abs_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegBallPoint >
  regBall ()
  {
    if ( type_ == RCC_RegPointUnion::REG_BALL )
      return reg_ball_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegBallPoint >
  regBall () const
  {
    if ( type_ == RCC_RegPointUnion::REG_BALL )
      return reg_ball_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegPlayerPoint >
  regPlayer ()
  {
    if ( type_ == RCC_RegPointUnion::REG_PLAYER )
      return reg_player_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegPlayerPoint >
  regPlayer () const
  {
    if ( type_ == RCC_RegPointUnion::REG_PLAYER )
      return reg_player_p;
    return NULL;
  }

  RCC_Pointer < RCC_RegRelPoint >
  regRel ()
  {
    if ( type_ == RCC_RegPointUnion::REG_REL )
      return reg_rel_p;
    return NULL;
  }

  const RCC_Pointer < RCC_RegRelPoint >
  regRel () const
  {
    if ( type_ == RCC_RegPointUnion::REG_REL )
      return reg_rel_p;
    return NULL;
  }
 
  RCC_Pointer < RCC_RegPoint >
  reg ()
  {
    return reg_p;
  }

  const RCC_Pointer < RCC_RegPoint >
  reg () const
  {
    return reg_p;
  }



private:
  RCC_Pointer < RCC_RegPoint > reg_p; // used to keep ref count above zero
  union
  {
    RCC_RegAbsPoint*	reg_abs_p;
    RCC_RegBallPoint*	reg_ball_p;
    RCC_RegPlayerPoint*	reg_player_p;
    RCC_RegRelPoint*	reg_rel_p;
  };

  RCC_RegPointUnion::type_t	type_;
};



class RCC_RegAbsPoint 
  : public RCC_RegPoint
{
public:
  RCC_RegAbsPoint() {}
  RCC_RegAbsPoint( const RCC_Vector2d& p ) : vec ( p ) {}
  ~RCC_RegAbsPoint() {}

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(pt " << vec.x () << " " << vec.y () << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const 
  { return printSimple ( out << line_header ) << std::endl; }
  
  std::ostream& 
  printSimple ( std::ostream& out ) const
  { return out << "Point(" << vec.x () << ", " << vec.y () << ")"; }

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_RegAbsPoint ( vec ); }

  RCC_Vector2d vector2d () const { return vec; }
  RCC_Vector2d& vector2d ( const RCC_Vector2d& v ) { return vec = v; }
  
private:
  RCC_Vector2d vec;
} ;



class RCC_RegObjPoint 
  : public RCC_RegPoint
{
public:
  virtual ~RCC_RegObjPoint () {}
};



class RCC_RegBallPoint 
  : public RCC_RegObjPoint
{
public:
  RCC_RegBallPoint () {}
  ~RCC_RegBallPoint () {}


  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(pt ball)"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const 
  { return printSimple ( out << line_header ) << std::endl; }
  
  std::ostream& 
  printSimple ( std::ostream& out ) const
  { return out << "Point-Ball"; }

  RCC_Pointer < RCC_Object > copy () const { return new RCC_RegBallPoint (); } 
};



class RCC_RegPlayerPoint 
  : public RCC_RegObjPoint
{
public:
  RCC_RegPlayerPoint ( const RCC_Side& s, const int& u ) : side_ ( s ), unum ( u ) {}
  ~RCC_RegPlayerPoint () {}

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(pt " << side_ << " " << unum << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const 
  { return printSimple ( out << line_header ) << std::endl; }
  
  std::ostream& 
  printSimple ( std::ostream& out ) const
  { return out << "Point-Player(" << side_ << " " << unum << ")"; }

  RCC_Pointer < RCC_Object > 
  copy () const 
  { return new RCC_RegPlayerPoint ( side_, unum ); }

  RCC_Side side () { return side_; }
  RCC_Side side ( const RCC_Side& s ) { return side_ = s; }

  int uNum () const { return unum; }

private:
  RCC_Side	side_;
  int		unum;
};



class RCC_RegRelPoint 
  : public RCC_RegAbsPoint
{
public:
  RCC_RegRelPoint ( const RCC_Vector2d& v, 
		    RCC_Pointer < RCC_RegPointUnion > p )
    : 
    RCC_RegAbsPoint ( v ), 
    point_p ( p ) {}
  
  ~RCC_RegRelPoint () {}

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "(pt " << vector2d ().x () << " " << vector2d ().y () 
	       << " " << *( point ()->reg () ) << ")"; }
  
  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const 
  { return printSimple ( out << line_header ) << std::endl; }

  std::ostream& 
  printSimple ( std::ostream& out ) const
  { return point ()->reg ()->printSimple ( out << "Point-Relative(" 
					   << vector2d ().x () << " " 
					   << vector2d ().y () ) << ")"; }

  RCC_Pointer < RCC_Object > copy () const 
  { 
    RCC_Pointer < RCC_RegPointUnion > point_copy;
    point_copy.staticCast ( point_p->copy () );
    return new RCC_RegRelPoint ( vector2d (), point_copy ); 
  }
  
  RCC_Pointer < RCC_RegPointUnion >
  point () 
  { return point_p; }
  
  const RCC_Pointer < RCC_RegPointUnion >
  point () const 
  { return point_p; }

protected:
  RCC_Pointer < RCC_RegPointUnion > point_p;
};



class RCC_RegQuad 
  : public RCC_Region
{
public:
  RCC_RegQuad ( const RCC_Pointer < RCC_RegPointUnion >& pt0, 
		const RCC_Pointer < RCC_RegPointUnion >& pt1, 
		const RCC_Pointer < RCC_RegPointUnion >& pt2, 
		const RCC_Pointer < RCC_RegPointUnion >& pt3 ) 
  {
    pts[0] = pt0;  
    pts[1] = pt1;
    pts[2] = pt2;
    pts[3] = pt3;
  }
  ~RCC_RegQuad() {}

  std::ostream& print ( std::ostream& out ) const;
  std::ostream& printPretty ( std::ostream& out, 
			      const RCC_String& line_header ) const;

  /* class specific */
  RCC_Pointer < RCC_RegPointUnion >
  point ( const unsigned int& i )
  { 
    if ( i < 4 )
      return ( pts [ i ] );
    else
      throw RCC_BoundsException ( "Point idx out of bounds" );
  }

  const RCC_Pointer < RCC_RegPointUnion >
  point ( const unsigned int& i ) const 
  { 
    if ( i < 4 )
      return ( pts [ i ] );
    else
      throw RCC_BoundsException ( "Point idx out of bounds" );
  }

  void point ( const unsigned int& i, 
	       const RCC_Pointer < RCC_RegPointUnion >& pt)
  {
    if ( i < 4 )
      pts [ i ] = pt;
    else
      throw RCC_BoundsException ( "Point idx out of bounds" );
  }

  RCC_Pointer < RCC_Object > copy() const 
  { 
    RCC_Pointer < RCC_RegPointUnion > pts_copy [ 4 ];
    for ( int i = 0; i < 4; i++ )
      pts_copy [ i ].staticCast ( pts [ i ]->copy () );
    
    return new RCC_RegQuad ( pts_copy [ 0 ], 
			     pts_copy [ 1 ], 
			     pts_copy [ 2 ], 
			     pts_copy [ 3 ] ); 
  }

protected:
  RCC_Pointer < RCC_RegPointUnion > pts[4];
};




class RCC_RegArc 
  : public RCC_Region
{
public:
  RCC_RegArc( const RCC_Pointer < RCC_RegPointUnion >& c, 
	      const double& sr, 
	      const double& er, 
	      const RCC_Angle& start_a, 
	      const RCC_Angle& span_a )
    : 
    center_p ( c ), 
    start_rad ( sr ), 
    end_rad ( er ), 
    start_ang ( start_a ), 
    span_ang ( span_a ) 
  {
    if ( start_rad > end_rad )
      throw RCC_GeomException ( "Start radius greater than end radius" );
  } 
  ~RCC_RegArc() {}

  std::ostream& 
  print ( std::ostream& out ) const;

  std::ostream& 
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  /* class specific */
  RCC_Pointer < RCC_RegPointUnion >
  center () 
  { return center_p; }
  
  const RCC_Pointer < RCC_RegPointUnion >
  center () const 
  { return center_p; }
  
  double 
  startRad () const 
  { return start_rad; }
  
  double 
  endRad () const
  { return end_rad; }
  
  RCC_Angle
  startAng () const 
  { return start_ang; }
  
  RCC_Angle 
  spanAng () const
  { return span_ang; }

  RCC_Pointer < RCC_RegPointUnion > 
  center ( RCC_Pointer < RCC_RegPointUnion > pt ) 
  { return center_p = pt; }
  
  void 
  rad ( const double& sr, 
	const double& er );
  
  void 
  ang ( const RCC_Angle& start_a, 
	const RCC_Angle& span_a );

  RCC_Pointer < RCC_Object > copy() const 
  {
    RCC_Pointer < RCC_RegPointUnion > point_copy;
    point_copy.staticCast ( center_p->copy () );
    return new RCC_RegArc ( point_copy, 
			    start_rad, 
			    end_rad, 
			    start_ang, 
			    span_ang ); 
  }
  
protected:
  RCC_Pointer < RCC_RegPointUnion > center_p;
  /* start rad <= end_rad */
  double start_rad, end_rad;
  RCC_Angle start_ang, span_ang;
} ;




class RCC_RegMerged
  : public RCC_Region

{
public:
  RCC_RegMerged () {}
  ~RCC_RegMerged() {}

  std::ostream& 
  print ( std::ostream& out ) const;
  
  std::ostream&
  printPretty ( std::ostream& out, 
		const RCC_String& line_header ) const;

  /* class specific */
  RCC_RegUnion::container& 
  regions () 
  { return regions_; }
  
  const RCC_RegUnion::container&
  regions () const 
  { return regions_; }

  RCC_Pointer < RCC_Object >
  copy() const 
  {
    RCC_Pointer < RCC_RegMerged > merged_copy = new RCC_RegMerged (); 
    for ( RCC_RegUnion::container::const_iterator i = regions_.begin ();
	  i != regions_.end (); ++i )
      {
	merged_copy->add ( ( *i )->copy ().staticCast < RCC_RegUnion > () );
      }
    return merged_copy; 
  }
  
  void
  add ( RCC_Pointer < RCC_RegUnion > reg )
  {
    regions_.push_back ( reg );
  }

protected:
  RCC_RegUnion::container regions_;
} ;


class RCC_RegNamed
  : public RCC_Region
{
public:
  RCC_RegNamed ( const RCC_String& n ) : name_ ( n ) {}
  ~RCC_RegNamed () {}

  std::ostream& 
  print ( std::ostream& out ) const
  { return out << "\"" << name_ << "\""; }

  std::ostream& printPretty ( std::ostream& out, 
			      const RCC_String& line_header) const
  { return out << line_header << "region named \"" << name_ 
	       << "\"" << std::endl; }

  /* class specific */
  RCC_String name () { return name_; }
  void  name ( const RCC_String& n ) { name_ = n; }

  RCC_Pointer < RCC_Object > copy() const { return new RCC_RegNamed ( name_ ); }

protected:
  RCC_String name_;
} ;





class RCC_BallMoveSet : public RCC_Object
{
public:
  enum RCC_BallMove_t { None, Pass, Dribble, Clear, Score, All };
  static const RCC_String NAMES[];
  static const unsigned int MAX;

  RCC_BallMoveSet ();
  RCC_BallMoveSet ( const RCC_BallMoveSet::RCC_BallMove_t& init_entry );
  ~RCC_BallMoveSet ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_BallMoveSet ( *this ); }

  void clear() { entries = 0; }
  void add ( const RCC_BallMoveSet::RCC_BallMove_t& t );
  void remove ( const RCC_BallMoveSet::RCC_BallMove_t& t );
  bool isMember ( const RCC_BallMoveSet::RCC_BallMove_t& t );

  std::ostream& print ( std::ostream& out ) const;
  friend std::ostream& operator <<(std::ostream & os, const RCC_BallMoveSet& s )
    { return s.print(os); }
 
protected:
  unsigned entries;
};

class RCC_UnumSet : public RCC_Object
{
public:
  RCC_UnumSet ( int ts = MAX_PLAYER );
  ~RCC_UnumSet ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_UnumSet ( *this ); }

  static const unsigned int ALL;

  void clear() { entries = 0; }
  void add ( const unsigned int& i );
  void remove ( const unsigned int& i );
  bool isMember ( const unsigned int& i ) const;

  std::ostream& print ( std::ostream& out ) const;
  friend std::ostream& operator <<( std::ostream & os, const RCC_UnumSet& s )
    { return s.print ( os ); }

 
protected:
  unsigned int team_size;
  unsigned int entries;
};


#endif
