// -*-c++-*-

/***************************************************************************
                          rccside.h  -  Interface for a basic class to
			                repesenent is something is on our
					or an opponents side.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_SIDE_H_
#define _RCC_SIDE_H_

#pragma interface

#include "param.h"
#include "types.h"

#include "rccstring.h"
#include <iostream>
#include "rccobject.h"
#include "rccmemory.h"

class RCC_Side : public RCC_Object
{
public:
  enum mode_t { Unknown, Our, Opp, Neut };
  static const RCC_String NAMES [];

  RCC_Side () : mode_ ( RCC_Side::Unknown ) {}
  RCC_Side ( const RCC_Side::mode_t& m ) : mode_ ( m ) {}
  RCC_Side ( const Side& s, const Side& team_side ) { mode ( s, team_side ); }
  ~RCC_Side () {}

  RCC_Side::mode_t mode () const { return mode_; }
  RCC_Side::mode_t mode ( const RCC_Side::mode_t m ) { return mode_ = m; }
  
  Side stadMode ( const Side& team_side ) const
  {
    switch ( mode_ )
      {
      case RCC_Side::Unknown:
      case RCC_Side::Neut:
	return NEUTRAL;

      case RCC_Side::Our:
	return team_side;

      case RCC_Side::Opp:
	return team_side * -1;
      }

    // Default case, should never happen!
    return NEUTRAL;
  }

  RCC_Side::mode_t mode ( const Side& s, const Side& team_side ) 
  {
    switch ( s )
      {
      case NEUTRAL:
	return mode_ = RCC_Side::Neut;
      case LEFT:
	switch ( team_side )
	  {
	  case LEFT:
	    return mode_ = RCC_Side::Our;
	  case RIGHT:
	    return mode_ = RCC_Side::Opp;
	  default:
	    return mode_ = RCC_Side::Unknown;
	  }
      case RIGHT:
	switch ( team_side )
	  {
	  case LEFT:
	    return mode_ = RCC_Side::Opp;
	  case RIGHT:
	    return mode_ = RCC_Side::Our;
	  default:
	    return mode_ = RCC_Side::Unknown;
	  }
      default:
	return mode_ = RCC_Side::Unknown;
      }
  }

  friend std::ostream& operator<< ( std::ostream& o, const RCC_Side& s )
  {
    return s.print ( o );
  }

  std::ostream& print ( std::ostream& o ) const
  {
    return o << RCC_Side::NAMES [ mode_ ];
  }

  std::ostream& print ( std::ostream& o, const Side& team_side ) const
  {
    switch ( stadMode ( team_side ) )
      {
      case LEFT:
	return o << "l";
      case RIGHT:
	return o << "r";
      default:
	return print ( o );
      }
  }

  bool ourSide () const { return mode_ == RCC_Side::Our; }
  bool oppSide () const { return mode_ == RCC_Side::Opp; }
  bool neutSide () const { return mode_ == RCC_Side::Neut; }

  RCC_Pointer < RCC_Object > copy () const { return new RCC_Side ( *this ); }

private:
  RCC_Side::mode_t mode_;
};

#endif
