// -*-c++-*-

/***************************************************************************
                          rcctransqueue.h  -  Interface and implemetaion of
			                      a thread safe queue to store
					      transmissions.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_TRANS_QUEUE_H_
#define _RCC_TRANS_QUEUE_H_


#include <queue>
#include "rcchandler.h"
#include "rccsyncable.h"
#include "rccexception.h"

template < typename T >
class RCC_TransQueue
  : public RCC_Handler < T >,
    public RCC_Syncable
{
public:
  void handle ( const T& t )
  {
    this->lock ();
    transmissions.push ( t );
    this->preLockedSignal ();
    this->unlock ();
  }

  T front ( const bool& pop = false )
  {
    this->lock ();
    if ( transmissions.size () == 0 )
      this->preLockedWait ();
    if ( transmissions.size () == 0 )
      throw RCC_Exception ( "No items in RCC_TransQueue" );

    T tmp = transmissions.front ();
    if ( pop )
      transmissions.pop ();
    this->unlock ();
    return tmp;
  }

  T frontAndPop ()
  {
    return front ( true );
  }

private:
  std::queue < T > transmissions;
};

#endif
