// -*-c++-*-
#ifndef _RCC_VIEWMODE_H_
#define _RCC_VIEWMODE_H_

#pragma interface

#include <iostream>
#include "rccstring.h"
#include "rccmemory.h"
#include "rccobject.h"

class RCC_ViewMode : public RCC_Object
{
public:
  enum width_t { NARROW, NORMAL, WIDE };
  enum quality_t { HIGH, LOW };

  static const double WIDTH_MODIFIERS[];
  static const RCC_String QUALITY_NAMES[];
  static const RCC_String WIDTH_NAMES[];
    
  RCC_ViewMode ( const RCC_ViewMode::quality_t &q, const RCC_ViewMode::width_t &w );
  ~RCC_ViewMode ();

  RCC_Pointer < RCC_Object > copy () const { return new RCC_ViewMode ( *this ); }

  double widthMod () const { return RCC_ViewMode::WIDTH_MODIFIERS [ wid ]; }

  RCC_ViewMode::width_t width () const { return wid; }
  RCC_ViewMode::width_t width ( const RCC_ViewMode::width_t& w ) { return ( wid = w ); }

  RCC_ViewMode::quality_t quality () const { return qual; }
  RCC_ViewMode::quality_t quality ( const RCC_ViewMode::quality_t& q ) { return ( qual = q ); }

  std::ostream& print ( std::ostream& o ) const; 
  friend std::ostream& operator<< (std::ostream& o, const RCC_ViewMode& v) { return v.print ( o ); }

  friend bool operator== ( const RCC_ViewMode& a, const RCC_ViewMode& b )
  {
    return ( a.width () == b.width () && a.quality () == b.quality () );
  }

private:
  RCC_ViewMode::quality_t qual;
  RCC_ViewMode::width_t wid;
    
};


std::ostream& operator<< (std::ostream& o, const RCC_ViewMode::width_t& w);
std::ostream& operator<< (std::ostream& o, const RCC_ViewMode::quality_t& q);

#endif
