// -*-c++-*-

/***************************************************************************
                          rccvisual.h  -  Interface for RoboCup visual
			                  (or "see") messages.
                             -------------------
    begin                : 17-JULY-2001
    copyright            : (C) 2001 by Tom Howard.
    email                : thomashoward@email.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _RCC_VISUAL_H_
#define _RCC_VISUAL_H_

#pragma interface

#include "rccvisualobject.h"
#include <vector>
#include <iostream>
#include "rccsensordata.h"
#include "rccmemory.h"
#include "rccstring.h"
#include "rccexception.h"

class RCC_Visual : public RCC_SensorData
{
public:
  RCC_Visual ( const RCC_String& tn );
  ~RCC_Visual ();
  
  RCC_Pointer < RCC_Object > copy () const 
  { 
    RCC_Pointer < RCC_VisualObject::container > objs_copy;
    objs_copy.staticCast ( objects->copy () );
    return new RCC_Visual ( team_name ); 
   }

   int time () const { return op_time; }
   int time ( const int &time ) { return op_time = time; }

   void add ( RCC_Pointer < RCC_Line > line );
   void add ( RCC_Pointer < RCC_Flag > flag );
   void add ( RCC_Pointer < RCC_Player > player );
   void add ( RCC_Pointer < RCC_Ball > ball );

   friend std::ostream& operator<<(std::ostream& o, const RCC_Visual& op) { return op.print(o); }
   std::ostream& print ( std::ostream& o ) const
   {
     return print ( o, RCC_Vector2d::POLAR, RCC_Angle::DEG );
   }
   std::ostream& print ( std::ostream& o, const Side& ) const { return print ( o ); } 

   std::ostream& print ( std::ostream& o, RCC_Vector2d::mode_t vm, RCC_Angle::mode_t am ) const;

   unsigned int numOfBalls () const { return balls->size (); } 
   const RCC_Pointer < RCC_Ball >& ball ( unsigned int idx = 0 ) const 
   { 
     if ( idx < numOfBalls () ) 
       return ( *balls ) [ idx ]; 
     throw RCC_BoundsException ( "Ball idx out of bounds" );
   }

   unsigned int numOfLines () const { return lines->size (); } 
   const RCC_Pointer < RCC_Line >& line ( unsigned int idx = 0 ) const 
   { 
     if ( idx < numOfLines () ) 
       return ( *lines ) [ idx ]; 
     throw RCC_BoundsException ( "Line idx out of bounds" );
   }

   unsigned int numOfFlags () const { return flags->size (); } 
   const RCC_Pointer < RCC_Flag >& flag ( unsigned int idx = 0 ) const 
   { 
     if ( idx < numOfFlags () ) 
       return ( *flags ) [ idx ]; 
     throw RCC_BoundsException ( "Flag idx out of bounds" ); 
   }

   unsigned int numOfGoals () const { return goals->size (); } 
   const RCC_Pointer < RCC_Flag >& goal ( unsigned int idx = 0 ) const 
   { 
     if ( idx < numOfGoals () ) 
       return ( *goals ) [ idx ]; 
     throw RCC_BoundsException ( "Goal idx out of bounds" ); 
   }


   unsigned int numOfPlayers () const { return numOfAllies () + numOfNeuts () + numOfEnemies (); } 
   const RCC_Pointer < RCC_Player >& player ( unsigned int idx = 0 ) const 
   { 
     if ( idx < numOfAllies () ) 
       return ally ( idx );

     else if ( idx - numOfAllies () < numOfNeuts () )
       return neut ( idx - numOfAllies () );

     else if ( idx - numOfAllies () - numOfNeuts () < numOfEnemies () )
       return enemy ( idx - numOfAllies () - numOfNeuts () );

     throw RCC_BoundsException ( "Player idx out of bounds" ); 
   }

   unsigned int numOfAllies () const { return allies->size (); } 
   const RCC_Pointer < RCC_Player >& ally ( unsigned int idx = 0 ) const 
   { 
     if ( idx < numOfAllies () ) 
       return ( *allies ) [ idx ]; 
     throw RCC_BoundsException ( "Ally idx out of bounds" ); 
   }

  unsigned int numOfNeuts () const { return neuts->size (); } 
  const RCC_Pointer < RCC_Player >& neut ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfNeuts () ) 
      return ( *neuts ) [ idx ]; 
    throw RCC_BoundsException ( "Neut idx out of bounds" ); 
  }

  unsigned int numOfEnemies () const { return enemies->size (); } 
  const RCC_Pointer < RCC_Player >& enemy ( unsigned int idx = 0 ) const 
  { 
    if ( idx < numOfEnemies () ) 
      return ( *enemies ) [ idx ]; 
    throw RCC_BoundsException ( "Enemy idx out of bounds" ); 
  }

private:
  RCC_String team_name;
  int op_time;

  RCC_Pointer < RCC_VisualObject::container > objects;
    
  RCC_Pointer < RCC_Line::container > 	lines;	// 0, 1 or many
  RCC_Pointer < RCC_Flag::container >	flags;	// 0, 1 or many
  RCC_Pointer < RCC_Flag::container >	goals;	// 0, 1 or 2
  RCC_Pointer < RCC_Player::container > allies;	// 0, 1 or many
  RCC_Pointer < RCC_Player::container > enemies;// 0, 1 or many
  RCC_Pointer < RCC_Player::container > neuts;	// 0, 1 or many
  RCC_Pointer < RCC_Ball::container > 	balls;	// 0 or 1

  void _add ( RCC_Pointer < RCC_VisualObject > obj ) { objects->push_back ( obj ); }


};

#endif
